{{ vars.permissions.imports|generate_import }}
# from invenio_records_permissions.generators import SystemProcess, AnyUser

class {{ vars.permissions|class_header }}:
    """{{ vars.record.class }} permissions.
        Values in this class will override permission presets.
    """

    # can_search = [SystemProcess(), AnyUser()]
    # can_read = [SystemProcess(), AnyUser()]
    # can_create = [SystemProcess()]
    # can_update = [SystemProcess()]
    # can_delete = [SystemProcess()]
    # can_manage = [SystemProcess()]

{{ vars.permissions.extra_code }}